﻿using System;
using System.Data;
using System.Data.SqlClient;
using System.Data.SqlTypes;
using Microsoft.SqlServer.Server;
using System.IO;
using System.Collections;
using System.Text.RegularExpressions;
using System.Collections.Generic;

[Serializable]
[Microsoft.SqlServer.Server.SqlUserDefinedType(Format.UserDefined, MaxByteSize = 24)]
public struct Currency : INullable, IBinarySerialize
{
    public bool IsNull
    {
        get
        {
            return _null;
        }
    }

    public static Currency Null
    {
        get
        {
            Currency h = new Currency();
            h._null = true;
            return h;
        }
    }

    public override string ToString()
    {
        return _symbol + _value.ToString();
    }

    public static Currency Parse(SqlString s)
    {
        if (s.IsNull ||
            string.IsNullOrEmpty(s.Value) ||
            s.Value.Length < 2 ||
            string.Equals(s.Value, "NULL", StringComparison.OrdinalIgnoreCase))
        {
            return Currency.Null;
        }
        Currency u = new Currency();
        u._symbol = s.Value[0];
        u._value = Decimal.Parse(s.Value.Substring(1));
        return u;
    }

    public char Symbol
    {
        get
        {
            return _symbol;
        }
    }

    [SqlMethod(IsMutator = true)]
    public void ChangeSymbol(char newSymbol)
    {
        _symbol = newSymbol;
    }

    public static Currency CreateCurrency(char symbol, Decimal value)
    {
        Currency c = new Currency();
        c._symbol = symbol;
        c._value = value;
        return c;
    }

    private char _symbol;
    private Decimal _value;
    private bool _null;

    public void Read(BinaryReader r)
    {
        _null = r.ReadBoolean();
        _symbol = r.ReadChar();
        _value = r.ReadDecimal();
    }

    public void Write(BinaryWriter w)
    {
        w.Write(_null);
        w.Write(_symbol);
        w.Write(_value);
    }
}


