﻿using System;
using System.Data;
using System.Data.SqlClient;
using System.Data.SqlTypes;
using Microsoft.SqlServer.Server;
using System.Collections;
using System.Text.RegularExpressions;
using System.Collections.Generic;

public partial class UserDefinedFunctions
{
    private struct _match
    {
        public readonly int MatchNumber;
        public readonly int GroupNumber;
        public readonly string CaptureValue;
        public _match(int matchNumber, int groupNumber, string captureValue)
        {
            MatchNumber = matchNumber;
            GroupNumber = groupNumber;
            CaptureValue = captureValue;
        }
    }

    [SqlFunction(FillRowMethodName = "FillRow",
                 TableDefinition = "MatchNumber int, " +
                                   "GroupNumber int, " +
                                   "CaptureValue nvarchar(MAX)")]
    public static IEnumerable GetCaptureGroupValues(string input, string pattern)
    {
        List<_match> matchList = new List<_match>();
        int matchIndex = 0;
        foreach (Match m in Regex.Matches(input, pattern))
        {
            int groupIndex = 0;
            foreach (Group g in m.Groups)
            {
                matchList.Add(new _match(matchIndex, groupIndex++, g.Value));
            }
            matchIndex++;
        }
        return matchList;
    }

    public static void FillRow(Object obj,
                               out int MatchNumber,
                               out int GroupNumber,
                               out SqlString CaptureValue)
    {
        _match match = (_match)obj;
        MatchNumber = match.MatchNumber;
        GroupNumber = match.GroupNumber;
        CaptureValue = match.CaptureValue;
    }
};

